/***************************************************************************************
 *																													*
 *	SPI_ADAPTER_PLUGIN.V:	Top level plug-in for the GR8BUS master board for the set	*
 *										of SPI-bus devices attached to Altera Cyclone III chip	*
 *																						http://www.gr8bit.ru	*
 *	Copyleft (cl) 2012 by Eugeny Brychkov																*
 *																													*
 *									License and T&C															*
 *																													*
 * You can use and modify this software's source and executable, given two					*
 * conditions are met: (a) you keep this Copyleft statement in the source code,			*
 * and you do not distribute software and resulting executables on the commercial		*
 * basis and for the purposes, not related to GR8BIT platform and/or MSX standard,		*
 * without written permission of Eugeny Brychkov (eugeny_brychkov@hotmail.com)			*
 *																													*
 ***************************************************************************************
 *			File name: SPI_ADAPTER_PLUGIN.V																*
 **************************************************************************************/

 /* Release date: Dec 20, 2012
  
	Techical information:
	- it is an adapter to connect top level GR8BUS master configuration to the devices
	  attached to the SPI bus - LAN module, SD-card module and USB module
	- use "include" directive to attach to the project, and modify/comment BD/DIO/BWAIT_N
	  wires accordingly
*/

//-------------------------------------------------------------------------------------
// 1. Definitions and wire states
//-------------------------------------------------------------------------------------
wire LAN_module_cs = (BA[7:2] == 6'b1010_11) & ~BIORQ_N & BM1_N;		// LAN module access signals, I/O ports 0ach, 0adh and 0aeh
wire LAN_data_direction, LAN_wait_signal;										// GR8BUS direction, system WAIT signal
wire LAN_SPI_SCLK, LAN_SPI_CS_n, LAN_SPI_MOSI, LAN_SPI_MISO;			// SPI wires to connect LAN module and SPI arbiter
wire LAN_SPI_request, LAN_timeslot;												// control wires to connect LAN module and SPI arbiter

reg [7:0] LAN_data;																	// output LAN data
assign BD[7:0] = (LAN_module_cs & ~BRD_N) ? LAN_data : {8{1'bz}};		// GR8BUS data bus write access arbitration
assign m_dio = LAN_data_direction;												// GR8BUS data bus buffer direction control (active high = read op)
assign BWAIT_N = LAN_wait_signal;												// GR8BUS system bus WAIT signal

LAN_SPI_module W5100(
	.clk_2xSPI_clock		(I_BVDPCLK),
	.reset_n					(BRESET_N),
	.clk_sysbus_clock		(I_BCLOCK),
	.i_LAN_cs				(LAN_module_cs),
	.i_LAN_portnum			(BA[1:0]),
	.i_rd_n					(BRD_N),
	.i_wr_n					(BWR_N),
	.i_data					(BD[7:0]),
	.o_direction			(LAN_data_direction),
	.o_data					(LAN_data[7:0]),
	.o_wait_signal_n		(LAN_wait_signal),
	.o_LAN_SPI_request	(LAN_SPI_request),
	.i_LAN_timeslot		(LAN_timeslot),
	.o_LAN_SPI_SCLK		(LAN_SPI_SCLK),
	.o_LAN_SPI_CS_n		(LAN_SPI_CS_n),
	.o_LAN_SPI_MOSI		(LAN_SPI_MOSI),
	.i_LAN_SPI_MISO		(LAN_SPI_MISO)
);

SPI_arbiter SPI_arbiter(
	// arbiter control signals
	.arbiter_clock		(I_BVDPCLK),
	.reset_n			(BRESET_N),
	// LAN module control signals
	.i_LAN_SPIrequest	(LAN_SPI_request),
	.o_LAN_timeslot	(LAN_timeslot),
	.i_LAN_clock		(LAN_SPI_SCLK),
	.i_LAN_cs		(LAN_SPI_CS_n),
	.i_LAN_mosi		(LAN_SPI_MOSI),
	.o_LAN_miso		(LAN_SPI_MISO),

	.i_SD_SPIrequest	(1'b0),				// SD-card and USB devices are not implemented, always inactive
	.i_USB_SPIrequest	(1'b0),
	.i_SD_clock			(1'b0),
	.i_USB_clock		(1'b0),
	
	// SPI bus access pins
	.p_LANCS_n		(LANCS_N),
	.p_SDCS_n		(SDCS_N),
	.p_USBCS_n		(USBCS_N),
	.p_sclk			(SCLK),
	.p_mosi			(MOSI),
	.p_miso			(MISO)
);

/*********************************************************************************
 *			SPI_ADAPTER_PLUGIN.V																		*
 ********************************************************************************/
