;****************************************************************************************
;*											*
;*	This set of routines is the part of GR8BIT's Micro-SD card browsing and		*
;*							file download program		*
;*								http://www.gr8bit.ru	*
;*	Copyleft (cl) 2013 Eugeny Brychkov						*
;*											*
;*				License and T&C						*
;*											*
;*	You can use and modify its source and executable, given two conditions are	*
;*	met: (a) you keep this Copyleft statement in the source code, and you do	*
;*	not distribute software and resulting executable on the commercial basis	*
;*	and on form the purposes, not related to GR8BIT platform, without written	*
;*	permission of Eugeny Brychkov (eugeny_brychkov@hotmail.com)			*
;*											*
;*	This driver works with storage subsystem built on PIC18F46K22 microcontroller,	*
;*	and supports high density floppy disks in various configurations (DPB is	*
;*	dynamically calculated basing on boot sector), but with maximum of 9 sectors	*
;*	per FAT copy. Mode (and thus selection of RAM space allocated for FAT) is	*
;*	configured from the microcotnoller's EEPROM. FAT12 LVOLs can have up to 12	*
;*	FAT sectors in DOS1 and DOS2 mode. FAT16 LVOLs are only supported in DOS2 mode	*
;*											*
;****************************************************************************************
;*	File name: MICROSD.ASM								*
;****************************************************************************************

; The following assembler routines are used together with the abovementioned
;   MSX-BASIC program (software) to speed up the hardware access. The code below
;   has fixed address location (0c000h), however in BASIC program it is made
;   relocatable to the area dedicated to the user code.

;-----------------------------------------------------

; Sends command to the card from the buffer

	org	0c010h

SendCommand:
	ld	a,4		; 3e 04
	out	(0ach),a	; d3 ac
	ld	hl,Addr		; 21 00 c0	ABSOLUTE ADDRESS
	ld	b,6		; 06 06
Outer:
	push	bc		; c5
	ld	c,(hl)		; 4e	
	ld	d,128		; 16 80

	ld	b,8		; 06 08
Inner:
	ld	a,c		; 79
	ld	e,4		; 1e 04	
	and	d		; a2
	jr	nz,Not0		; 20 02
	ld	e,0		; 1e 00
Not0:
	ld	a,e		; 7b
	out	(0ach),a	; d3 ac
	inc	a		; 3c
	out	(0ach),a	; d3 ac
	srl	d		; cb 3a
	djnz	Inner		; 10 ee

	pop	bc		; c1
	inc	hl		; 23
	djnz	Outer		; 10 e4
	ret			; c9
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00, address 0c03fh

;-----------------------------------------------------

; Gets token/byte from the card

	org	0c040h

GetByte:	; (returns token in E)
	ld	de,8000h	; 11 00 80
	ld	b,8		; 06 08
Inner:	
	ld	a,4		; 3e 04
	out	(0ach),a	; d3 ac
	inc	a		; 3c
	out	(0ach),a	; d3 ac
	in	a,(0ach)	; db ac
	and	1		; e6 01
	jr	z,NoAdd		; 28 03
	ld	a,e		; 7b
	add	a,d		; 82
	ld	e,a		; 5f
NoAdd:
	srl	d		; cb 3a
	djnz	Inner		; 10 ec
	ret			; c9
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00, address 0c05fh

;-----------------------------------------------------

; Gets token from the card, preserve into buffer

	org	0c060h

GetToken:
	call	GetByte		; cd 40 c0	ABSOLUTE ADDRESS
	ld	a,e		; 7b
	ld	(0c006h),a	; 32 06 c0	ABSOLUTE ADDRESS
	ret			; c9
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00, address 0c06fh

;-----------------------------------------------------

; Gets R1 type response from the card

	org	0c070h

R1Response:
	ld	hl,200h		; 21 00 02
Outer:
	call	GetToken	; cd 60 c0	ABSOLUTE ADDRESS
	and	80h		; e6 80
	jr	z,Return	; 28 05

	dec	hl		; 2b
	ld	a,h		; 7c
	or	l		; b5
	jr	nz,Outer	; 20 f4
Return:
	ret			; c9
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00, address 0c087h

;-----------------------------------------------------

; Sends 8 spare SCLKs to card with chip select line active
; then deactivates chip select (valid MicroSD command end)

	org 0c088h

Send8:
	ld	b,8		; 06 08
SpareCycles:
	ld	a,4		; 3e 04
Loop:
	out	(0ach),a	; d3 ac
	inc	a		; 3c
	out	(0ach),a	; d3 ac
	dec	a		; 3d
	djnz	Loop		; 10 f8
	out	(0ach),a	; d3 ac
	ld	a,6		; 3e 06
	out	(0ach),a	; d3 ac
	ret			; c9
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00, address 0c09fh

;-----------------------------------------------------

; Sends 75 clocks to the card with chip select signal active

	org	0c0a0h

Send75:
	ld	b,75		; 06 4b
	call	SpareCycles	; cd 8a c0	ABSOLUTE ADDRESS
	ret			; c9
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00, address 0c0afh

;-----------------------------------------------------

; Transfer data from the card

	org	0c0b0h

DataTransfer:
	ld	hl,200h		; 21 00 02
DataWait:
	call	GetToken	; cd 60 c0	ABSOLUTE ADDRESS
	cp	0feh		; fe fe
	jr	z,DataTransfer	; 28 09
	cp	0ffh		; fe ff
	ret	nz		; c0

	dec	hl		; 2b
	ld	a,h		; 7c
	or	l		; b5
	jr	nz,DataWait	; 20 f1
	ret			; c9

DataTransfer:
	call	ReadData	; cd d8 c0	ABSOLUTE ADDRESS
	; now CRC
	call	GetByte		; cd 40 c0	ABSOLUTE ADDRESS
	ld	l,a		; 6f
	call	GetByte		; cd 40 c0	ABSOLUTE ADDRESS
	ld	h,a		; 67
	ld	(0c00ah),hl	; 22 0a c0	ABSOLUTE ADDRESS
	ret			; c9
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00, address 0c0d7h

;-----------------------------------------------------

; Gets data from card to the $SECBUF

	org	0c0d8h

ReadData:
	ld	bc,(0c008h)	; ed 4b 08 c0	ABSOLUTE ADDRESS
	ld	hl,(0f34dh)	; 2a 4d f3
Transfer:
	push	bc		; c5
	call	GetByte		; cd 40 c0	ABSOLUTE ADDRESS
	ld	(hl),e		; 73
	inc	hl		; 23
	pop	bc		; c1
	dec	bc		; 0b
	ld	a,b		; 78
	or	c		; b1
	jr	nz,Transfer	; 20 f4
	ret			; c9
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00, address 0c0efh

;-----------------------------------------------------

; Create CRC7 checksum for SPI command, store to buffer

	org	0c0f0h

CRC7:
	ld	hl,0c000h	; 21 00 c0	ABSOLUTE ADDRESS
	xor	a		; af
	ld	b,5		; 06 05
Outer:
	ld	c,(hl)		; 4e
	push	bc		; c5

	ld	b,8		; 06 08
Inner:
	sla	a		; cb 27
	push	af		; f5
	xor	c		; a9
	ld	de,0801h	; 11 01 08
	and	80h		; e6 80
	jr	nz,Not0		; 20 03

	ld	de,0		; 11 00 00
Not0:
	pop	af		; f1
	xor	d		; aa
	add	a,e		; 83
	sla	c		; cb 21
	djnz	Inner		; 10 eb

	pop	bc		; c1
	inc	hl		; 23
	djnz	Outer		; 10 e3
	rlca			; 07
	or	01h		; f6 01
	ld	(0c005h),a	; 32 05 c0	ABSOLUTE ADDRESS
	ret			; c9
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00, address 0c11fh

;-----------------------------------------------------

; Write chunk of data from $SECBUF to the file

	org	0c120h

SaveChunk:
	ld	hl,(0f23dh)	; 2a 3d f2
	push hl			; e5
	ld	hl,(0f34dh)	; 2a 4d f3
	ld	(0f23dh),hl	; 22 3d f2
	ld	de,xxxx		; 11 58 c1	ABSOLUTE ADDRESS
	ld	hl,xxxx		; 21 00 00
	ld	c,26h		; 0e 26
	call	0f37dh		; cd 7d f3

	pop	hl		; e1
	ld	(0f23dh),hl	; 22 3d f2
	ld	(0c006h),a	; 32 06 c0	ABSOLUTE ADDRESS
	ret			; c9
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00, address 0c143h

;-----------------------------------------------------

; Opens or closes file depending on value in register C

	org	0c144h

OpenCloseFile:
	ld	de,xxxx		; 11 58 c1	ABSOLUTE ADDRESS
	ld	c,xx		; 0e (16 for open, 10 for close)
	call	0f37dh		; cd 7d f3
	ld	(0c006h),a	; 32 06 c0	ABSOLUTE ADDRESS
	ret			; c9
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00
	nop			; 00, 0c157h

;-----------------------------------------------------

; FCB buffer

	org	0c158h
FCBBuf:
	db	25h dup (0)	; end address 0c17ch

;****************************************************************************************
;*	End of: MICROSD.ASM								*
;****************************************************************************************
