/***************************************************************************************
 *																													*
 *	MICROSD_PLUGIN.V:	Micro-SD card device plugin into SPI channel								*
 *																						http://www.gr8bit.ru	*
 *	Copyleft (cl) 2013 by Eugeny Brychkov																*
 *																													*
 *									License and T&C															*
 *																													*
 * You can use and modify this software's source and executable, given two					*
 * conditions are met: (a) you keep this Copyleft statement in the source code,			*
 * and you do not distribute software and resulting executables on the commercial		*
 * basis and for the purposes, not related to GR8BIT platform and/or MSX standard,		*
 * without written permission of Eugeny Brychkov (eugeny_brychkov@hotmail.com)			*
 *																													*
 ***************************************************************************************
 *			File name: MICROSD_PLUGIN.V																	*
 **************************************************************************************/

 /* Release date: Jan 03, 2013
  
	Techical information:
	- extremely simple implementation of the SPI channel between GR8BUS and Micro-SD device,
	  all the protocol is handled in the GR8BIT software
	- Access port used is 0ACh
	- In this implementation does not use SPI arbiter
*/


wire SD_module_cs = (BA[7:0] == 8'b1010_1100) & ~BIORQ_N & BM1_N;
wire SD_data_direction = SD_module_cs & ~BRD_N;
assign m_dio = SD_data_direction;
reg [7:0] SD_data;
assign BD[7:0] = (SD_module_cs & ~BRD_N) ? SD_data[7:0] : {8{1'bz}};
reg [7:0] SPI_reg;

assign SCLK = SPI_reg[0];
assign SDCS_N = SPI_reg[1];
assign MOSI = SPI_reg[2];

always @(posedge I_BCLOCK or negedge BRESET_N) begin
	if(!BRESET_N) SPI_reg[7:0] <= 8'b00000110;		// LSB 0 = SPI mode 0 (SCLK idle LOW)
	else begin
		if(SD_module_cs && ~BWR_N)
			SPI_reg[7:0] <= BD[7:0];
		if(SD_data_direction)
			SD_data[0] <= MISO;
	end
end

/***************************************************************************************
 *			End of: MICROSD_PLUGIN.V																		*
 **************************************************************************************/
